import json
import sys
import requests
from urllib.parse import urljoin
from ultru_query.base import BaseRequestsClass
from ultru_query.record_types import Types
from ultru_query.scores import Scores

class Query(BaseRequestsClass):
    def reset(self):
        self.data = []
        self.uri = ""
        self.body = {}
        self.last_evaluated_key = None
        self.count = 0
        self.retrieve_all_types = False
        self._count_only = False

    def handle_error(self, body):
        err = body.get('body', {}).get('err', '')
        message = body.get('message', '')
        error_type = body.get('errorType', '')
        error_msg = body.get('errorMessage', '')
        stack_trace = body.get('stackTrace', '')
        if (message):
            self.logger.error(f"{message}")
        elif (err):
            self.logger.error(f"{err}")
        elif error_type:
            self.logger.error(f"{error_type}: {error_msg}\n{stack_trace}")
        else:
            self.logger.error(f"No error message")

    def handle_json(self, body):
        inner_body = body.get('body', {})
        self.last_evaluated_key = inner_body.get('LastEvaluatedKey')
        self.count += inner_body.get('Count', 0)
        if self._count_only is False:
            self.data.extend(inner_body.get('Items', []))
        else:
            self.data = self.count
        self.logger.debug(f"Query returns {self.count} results")

    def next(self):
        self.body['last_evaluated_key'] = self.last_evaluated_key
        self.execute()
        return self

    def by_all(self, score=None):
        if score is None:
            self.set_uri('query/type')
        else:
            if not isinstance(score, Scores):
                self.logger.error("Invalid scores object, not of Scores")
            self.set_uri('query/score')
        self.retrieve_all_types = True
        return self

    def by_type(self, type_):
        if not isinstance(type_, Types):
            self.logger.error(f"Invalid type, not of type Types: {type(type_)}")
            return None
        self.set_uri('query/type')
        self.body["record_type"] = type_.value
        return self

    def by_score(self, score, type_):
        if not isinstance(score, Scores):
            self.logger.error("Invalid score, not of type Scores")
            return None
        self.set_uri('query/score')
        self.body["score_low"] = score.low
        self.body["score_high"] = score.high
        self.body["record_type"] = type_.value
        return self

    def set_count(self, counter):
        self.body['counter'] = counter
        return self

    def all(self):
        self.execute()
        while self.last_evaluated_key:
            self.next()
        self.logger.info(f"Retrieved {self.count} records")
        return self

    def store(self, stream):
        stream.write(json.dumps(self.data).encode('UTF-8'))
        self.reset()
        return self

    def count_only(self):
        self._count_only = True
        self.body['select'] = "count"
        return self